package conf

import (
	"bytes"
	"gitlab.wanzhuangkj.com/tush/xpkg/xerrors/xerror"
	"log"

	consulapi "github.com/armon/consul-api"
	"github.com/spf13/viper"
)

const (
	DefaultEndPoint   = "consul.default.svc.cluster.local:8500"
	DefaultPath       = "xmall/conf"
	DefaultToken      = "9a6bc59e-73d7-7af5-a669-818a200a9ded"
	DefaultConfigType = "yml"
)

// JwtOption set the jwt options.
type ConsulOption func(*ConsulParser)

func WithToken(token string) ConsulOption {
	return func(o *ConsulParser) {
		o.token = token
	}
}

func WithConfigType(configType string) ConsulOption {
	return func(o *ConsulParser) {
		o.configType = configType
	}
}

type ConsulParser struct {
	endpoint     string
	configPath   string
	configType   string
	token        string
	cfg          *consulapi.Config
	consulClient *consulapi.Client
}

func NewConsulParser(endpoint, configPath string, opts ...ConsulOption) (*ConsulParser, error) {
	if endpoint == "" {
		return nil, xerror.New("endpoint cannot be empty")
	}
	if configPath == "" {
		return nil, xerror.New("configPath cannot be empty")
	}
	o := &ConsulParser{endpoint: endpoint, configPath: configPath}
	for _, opt := range opts {
		opt(o)
	}
	ccfg := consulapi.Config{Address: endpoint, Token: o.token}
	o.cfg = &ccfg
	consulClient, err := consulapi.NewClient(&ccfg)
	if err != nil {
		log.Fatal(err)
	}
	o.consulClient = consulClient
	return o, nil
}

func (x *ConsulParser) Read(obj any) error {
	kv, _, err := x.consulClient.KV().Get(x.configPath, nil)
	if err != nil {
		return err
	}
	viper.SetConfigType(x.configType)
	err = viper.ReadConfig(bytes.NewBuffer(kv.Value))
	if err != nil {
		return err
	}
	err = viper.Unmarshal(obj)
	if err != nil {
		return err
	}
	return nil
}
